﻿using System;
using CodeStage.AntiCheat.ObscuredTypes;
using UnityEngine;

namespace RocketGM.PanStats
{
    public class ByteStat : BaseStat
    {
        public override string StringValue => Current.ToString();

        public ObscuredByte Min { get; private set; }
        public ObscuredByte Max { get; private set; }
        public ObscuredByte Current { get; private set; }

        public ByteStat(byte min, byte max, byte current, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Math.Clamp(current, Min, Max);
        }

        /// <summary>
        /// Sets min value to current value.
        /// </summary>
        /// <param name="min"></param>
        /// <param name="max"></param>
        /// <param name="syncRemote"></param>
        public ByteStat(byte min, byte max, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Min;
        }

        public override T Get<T>()
        {
            ValidateType(typeof(T));

            return (T)Convert.ChangeType((byte)Current, typeof(T));
        }

        public override bool Set<T>(T value)
        {
            ValidateType(typeof(T));

            byte newValue = Math.Clamp(Convert.ToByte(value), Min, Max);

            if (newValue == Current)
                return false;
            
            Current = newValue;

            return base.Set(newValue);
        }

        public override Type GetStatType()
        {
            return typeof(byte);
        }
    }
}
