﻿using System;
using CodeStage.AntiCheat.ObscuredTypes;
using UnityEngine;

namespace RocketGM.PanStats
{
    public class DateStat : BaseStat {

        public override string StringValue => Current.ToString();

        public ObscuredLong Min { get; private set; }
        public ObscuredLong Max { get; private set; }
        public ObscuredLong Current { get; private set; }

        public DateStat(long current, bool syncRemote = true) : base(syncRemote)
        {
            Min = DateTime.MinValue.Ticks;
            Max = DateTime.MaxValue.Ticks;
            Current = Math.Clamp(Convert.ToInt64(current), Min, Max);
        }

        public DateStat(bool syncRemote = true) : base(syncRemote)
        {
            Min = DateTime.MinValue.Ticks;
            Max = DateTime.MaxValue.Ticks;
            Current = Min;
        }
        public override T Get<T>()
        {
            ValidateType(typeof(T));

            return (T) Convert.ChangeType(new DateTime(Current), typeof(T)) ;
        }

        public override bool Set<T>(T value)
        {
            ValidateType(typeof(T));

            var date = Convert.ToDateTime(value);

            long newValue = Math.Min(Max, Math.Max(Min, Convert.ToInt64(date.Ticks)));

            if (newValue == Current)
                return false;

            Current = newValue;

            return base.Set(newValue);
        }

        public override Type GetStatType()
        {
            return typeof(DateTime);
        }


    }
}
