﻿using System;
using System.Globalization;
using CodeStage.AntiCheat.ObscuredTypes;
using UnityEngine;

namespace RocketGM.PanStats
{
    public class DoubleStat : BaseStat
    {
        public override string StringValue => Current.ToString(CultureInfo.InvariantCulture);
        public ObscuredDouble Min { get; private set; }
        public ObscuredDouble Max { get; private set; }
        public ObscuredDouble Current { get; private set; }

        public DoubleStat(double min, double max, double current, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Math.Clamp(current, Min, Max);
        }

        /// <summary>
        /// Sets min value to current value.
        /// </summary>
        /// <param name="min"></param>
        /// <param name="max"></param>
        /// <param name="syncRemote"></param>
        public DoubleStat(double min, double max, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Min;
        }

        public override T Get<T>()
        {
            ValidateType(typeof(T));

            return (T) Convert.ChangeType((double)Current, typeof(T));
        }

        public override bool Set<T>(T value)
        {
            ValidateType(typeof(T));

            double newValue = Math.Clamp(Convert.ToDouble(value), Min, Max);

            if (Math.Abs(newValue - Current) < Mathf.Epsilon)
                return false;

            Current = newValue;

            return base.Set(newValue);
        }

        public override Type GetStatType()
        {
            return typeof(double);
        }
    }
}
