﻿using System;
using System.Globalization;
using CodeStage.AntiCheat.ObscuredTypes;
using UnityEngine;

namespace RocketGM.PanStats
{
    public class FloatStat : BaseStat
    {
        public override string StringValue => Current.ToString(CultureInfo.InvariantCulture);

        public ObscuredFloat Min { get; private set; }
        public ObscuredFloat Max { get; private set; }
        public ObscuredFloat Current { get; private set; }

        public FloatStat(float min, float max, float current, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Mathf.Clamp(current, Min, Max);
        }

        /// <summary>
        /// Sets min value to current value.
        /// </summary>
        /// <param name="min"></param>
        /// <param name="max"></param>
        /// <param name="syncRemote"></param>
        public FloatStat(float min, float max, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Min;
        }

        public override T Get<T>()
        {
            ValidateType(typeof(T));

            return (T) Convert.ChangeType((float)Current, typeof(T));
        }

        public override bool Set<T>(T value)
        {
            ValidateType(typeof(T));

            float newValue = Mathf.Clamp(Convert.ToSingle(value), Min, Max);

            if (Math.Abs(newValue - Current) < Mathf.Epsilon)
                return false;

            Current = newValue;

            return base.Set(newValue);
        }

        public override Type GetStatType()
        {
            return typeof(float);
        }
    }
}
