﻿using System;
using CodeStage.AntiCheat.ObscuredTypes;
using UnityEngine;

namespace RocketGM.PanStats
{
    public class LongStat : BaseStat
    {
        public override string StringValue => Current.ToString();

        public ObscuredLong Min { get; private set; }
        public ObscuredLong Max { get; private set; }
        public ObscuredLong Current { get; private set; }

        public LongStat(long min, long max, long current, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Math.Clamp(current, Min, Max);
        }

        /// <summary>
        /// Sets min value to current value.
        /// </summary>
        /// <param name="min"></param>
        /// <param name="max"></param>
        /// <param name="syncRemote"></param>
        public LongStat(long min, long max, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Min;
        }

        public override T Get<T>()
        {
            ValidateType(typeof(T));

            return (T)Convert.ChangeType((long) Current, typeof(T));
        }

        public override bool Set<T>(T value)
        {
            ValidateType(typeof(T));
			
			long newValue = Math.Clamp(Convert.ToInt64(value), Min, Max);

            if (newValue == Current)
                return false;

            Current = newValue;

            return base.Set(newValue);
        }

        public override Type GetStatType()
        {
            return typeof(long);
        }
    }
}
