﻿using System;
using RocketUtils;
using UnityEngine;

namespace RocketGM.PanStats
{
    public class ObjectStat<T1> : StringStat where T1 : class, IObservableModel, new()
    {
	    private T1 _objectModel;

	    private T1 ObjectModel
	    {
		    get { return _objectModel;}
		    set
		    {
			    if (_objectModel != null)
				    _objectModel.PropertyChanged -= OnPropertyChanged;
				    
			    _objectModel = value;
				    
			    _objectModel.PropertyChanged += OnPropertyChanged;
		    }
	    }

        public ObjectStat(string current, bool syncR) : base(current)
        {
            ObjectModel = string.IsNullOrEmpty(current) ? new T1() : JsonUtility.FromJson<T1>(Current);
        }

	    ~ObjectStat()
	    {
		    ObjectModel.PropertyChanged -= OnPropertyChanged;
		}

	    public override void Reconstruct(string value, bool syncRemote)
	    {
		    base.Reconstruct(value, syncRemote);

		    Current = value;

	        ObjectModel = string.IsNullOrEmpty(Current) ? new T1() : JsonUtility.FromJson<T1>(Current);
        }

	    private void OnPropertyChanged()
	    {
		    Current = JsonUtility.ToJson(ObjectModel);
		    IsDirty = true;
	    }

	    public override T Get<T>()
        {
	        if (typeof(T) != typeof(T1))
	        {
		        throw new InvalidCastException();
	        }

			return (T) Convert.ChangeType(ObjectModel, typeof(T));
		}

        public override bool Set<T>(T value)
        {
	        if (typeof(T) == typeof(T1))
	        {
		        return SerializeObject(value as T1);
	        }
	        
	        if (typeof(T) == typeof(string))
	        {
		        return ParseString(value as string);
	        }
	        
	        throw new InvalidCastException("Cannot cast value of type " 
	                                       + value.GetType().Name + " to type " 
	                                       + typeof(T1).Name);
        }

	    private bool SerializeObject(T1 obj)
	    {
		    if (Equals(obj, ObjectModel) || ReferenceEquals(obj, ObjectModel))
			    return false;
		    
		    Current = JsonUtility.ToJson(obj);

		    ObjectModel = obj;

		    IsDirty = true;

		    return true;
	    }

	    private bool ParseString(string value)
	    {
		    string newValue = Convert.ToString(value);
			 
		    if (newValue == Current)
			    return false;

		    Current = newValue;

		    ObjectModel = JsonUtility.FromJson<T1>(Current);

		    IsDirty = true;

		    return true;
	    }

		public override Type GetStatType()
        {
			return typeof(IObservableModel);
		}
    }
}
