﻿using System;
using CodeStage.AntiCheat.ObscuredTypes;
using UnityEngine;

namespace RocketGM.PanStats
{
    public class ShortStat : BaseStat
    {
        public override string StringValue => Current.ToString();

        public ObscuredShort Min { get; private set; }
        public ObscuredShort Max { get; private set; }
        public ObscuredShort Current { get; private set; }

        public ShortStat(short min, short max, short current, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Math.Clamp(current, Min, Max);
        }

        /// <summary>
        /// Sets min value to current value.
        /// </summary>
        /// <param name="min"></param>
        /// <param name="max"></param>
        /// <param name="syncRemote"></param>
        public ShortStat(short min, short max, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Min;
        }

        public override T Get<T>()
        {
            ValidateType(typeof(T));

            return (T)Convert.ChangeType((short)Current, typeof(T));
        }

        public override bool Set<T>(T value)
        {
            ValidateType(typeof(T));

            short newValue = Math.Clamp(Convert.ToInt16(value), Min, Max);

            if (newValue == Current)
                return false;
            
            Current = newValue;

            return base.Set(newValue);
        }

        public override Type GetStatType()
        {
            return typeof(short);
        }
    }
}
