﻿using System;
using CodeStage.AntiCheat.ObscuredTypes;
using UnityEngine;

namespace RocketGM.PanStats
{
    public class UIntStat : BaseStat
    {
        public override string StringValue => Current.ToString();

        public ObscuredUInt Min { get; private set; }
        public ObscuredUInt Max { get; private set; }
        public ObscuredUInt Current { get; private set; }

        public UIntStat(uint min, uint max, uint current, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Math.Clamp(current, Min, Max);
        }

        /// <summary>
        /// Sets min value to current value.
        /// </summary>
        /// <param name="min"></param>
        /// <param name="max"></param>
        /// <param name="syncRemote"></param>
        public UIntStat(uint min, uint max, bool syncRemote = true) : base(syncRemote)
        {
            Min = min;
            Max = max;
            Current = Min;
        }

        public override T Get<T>()
        {
            ValidateType(typeof(T));

            return (T)Convert.ChangeType((uint)Current, typeof(T));
        }

        public override bool Set<T>(T value)
        {
            ValidateType(typeof(T));

            uint newValue = Math.Clamp(Convert.ToUInt32(value), Min, Max);

            if (newValue == Current)
                return false;
            
            Current = newValue;

            return base.Set(newValue);
        }

        public override Type GetStatType()
        {
            return typeof(uint);
        }
    }
}
