﻿using System;
using System.Collections.Generic;
using RocketLog;

namespace RocketGM.PanStats
{
    public static class TempStats
    {
        private static Dictionary<string, BaseStat> _stats;

        private static readonly RocLog Log = new RocLog(nameof(PlayerStats), DebugLevels.Info);
        
        public static void Initialize(Dictionary<string, BaseStat> stats)
        {
            _stats = stats;
        }

        public static T Get<T>(string statKey)
        {
            return _stats[statKey].Get<T>();
        }

        public static T GetMin<T>(string statKey)
        {
            if (typeof(T) == typeof(int))
            {
                return (T)Convert.ChangeType(((IntStat) _stats[statKey]).Min, typeof(T));
            }
            if (typeof(T) == typeof(float))
            {
                return (T)Convert.ChangeType(((FloatStat)_stats[statKey]).Min, typeof(T));
            }
            if (typeof(T) == typeof(long))
            {
                return (T)Convert.ChangeType(((LongStat)_stats[statKey]).Min, typeof(T));
            }

            throw new InvalidCastException();
        }

        public static T GetMax<T>(string statKey)
        {
            if (typeof(T) == typeof(int))
            {
                return (T)Convert.ChangeType(((IntStat)_stats[statKey]).Max, typeof(T));
            }
            if (typeof(T) == typeof(float))
            {
                return (T)Convert.ChangeType(((FloatStat)_stats[statKey]).Max, typeof(T));
            }
            if (typeof(T) == typeof(long))
            {
                return (T)Convert.ChangeType(((LongStat)_stats[statKey]).Max, typeof(T));
            }

            throw new InvalidCastException();
        }

        public static void Set<T>(string statKey, T value)
        {
            try
            {
                _stats[statKey].Set(value);
            }
            catch (Exception e)
            {
                Log.Warning(statKey + " does NOT exist in DefaultPlayerStats.");
                throw;
            }
        }
    }
}
