﻿using System;
using System.Collections.Generic;
using RocketCoroutine;
using RocketLog;

namespace RocketGM.RemoteSettings
{
    public abstract class RocRemoteSettingsProvider
    {
        public event Action<bool> Initialized;
        public event Action<bool> RemoteSettingsReceived;

        protected readonly RocLog Log = new RocLog(nameof(RocRemoteSettingsProvider), DebugLevels.Info);

        public bool IsInitialized;
        public bool IsRemoteSettingsReceived;

        /// <summary>
        /// Calls given action if/when remote setings is initialized
        /// </summary>
        /// <param name="action"></param>
        public void DoAfterInitialized(Action action)
        {
            if (IsInitialized)
            {
                action();
            }
            else
            {
                Action<bool> eventHandler = null;
                eventHandler = delegate
                {
                    action();
                    Initialized -= eventHandler;
                };

                Initialized += eventHandler;
            }
        }

        protected void SetRemoteSettingsReceived(bool isSuccess = true)
        {
            IsRemoteSettingsReceived = isSuccess;
            if (RemoteSettingsReceived != null) RemoteSettingsReceived(isSuccess);

            // Wait for one frame to make sure that remote variables are updated
            // After that set initialize to true and fire event.
            CoroutineController.DoAfterFixedUpdate(() =>
            {
                IsInitialized = true;
                if (Initialized != null) Initialized(isSuccess);
            }, "gmRemoteSettingsInitialization", false);
        }

        public abstract bool HasKey(string key);

        public abstract int GetInt(string key, int defaultValue = 0);

        public abstract float GetFloat(string key, float defaultValue = 0);

        public abstract bool GetBool(string key, bool defaultValue = false);

        public abstract string GetString(string key, string defaultValue = "");

        public abstract List<T> GetArray<T>(string key, List<T> defaultValue);
    }
}
