﻿using System;
using UnityEngine;

namespace RocketGM
{
	[Serializable]
	public class RocGmGameSettings
	{
		public RocGmGameSettings() {}
				
		public RocGmGameSettings(string appStoreId, bool legacyMarketingEnabled, bool subscriptionEnabled, bool persistentProgressEnabled, bool remotePlayerDataEnabled)
		{
			this.appStoreId = appStoreId;
			this.subscriptionEnabled = subscriptionEnabled;
		    this.legacyMarketingEnabled = legacyMarketingEnabled;
		    PersistentProgressEnabled = persistentProgressEnabled;
			this.remotePlayerDataEnabled = remotePlayerDataEnabled;
		}

		public void CopyFrom(RocGmGameSettings gameSettings)
		{
			if (gameSettings.adjustFirstOpenEvent != null)
			{
				adjustFirstOpenEvent = gameSettings.adjustFirstOpenEvent;
			}

		    if (gameSettings.adjustFirstLevelCompletedEvent != null)
		    {
		        adjustFirstLevelCompletedEvent = gameSettings.adjustFirstLevelCompletedEvent;
		    }

		    if (gameSettings.adjustInterstitialAttemptEvent != null)
		    {
		        adjustInterstitialAttemptEvent = gameSettings.adjustInterstitialAttemptEvent;
		    }

		    if (gameSettings.adjustInterstitialShowEvent != null)
		    {
		        adjustInterstitialShowEvent = gameSettings.adjustInterstitialShowEvent;
		    }

		    if (gameSettings.adjustRewardedShowEvent != null)
		    {
		        adjustRewardedShowEvent = gameSettings.adjustRewardedShowEvent;
		    }

		    if (gameSettings.adjustPaymentInitEvent != null)
			{
				adjustPaymentInitEvent = gameSettings.adjustPaymentInitEvent;
			}

			if (gameSettings.adjustGiftEvent != null)
			{
				adjustGiftEvent = gameSettings.adjustGiftEvent;
			}
			
			if (gameSettings.supportLink != null)
			{
				supportLink = gameSettings.supportLink;
			}
			
			if (gameSettings.assetBundleBaseUrl != null)
			{
				assetBundleBaseUrl = gameSettings.assetBundleBaseUrl;
			}

			if (Math.Abs(gameSettings.minClientVersion) > Mathf.Epsilon)
			{
				minClientVersion = gameSettings.minClientVersion;
			}

			if (gameSettings.appStoreId != null)
			{
				appStoreId = gameSettings.appStoreId;
			}

            trackProgress = gameSettings.trackProgress;
			pushEnabled = gameSettings.pushEnabled;
			liveTuneEnabled = gameSettings.liveTuneEnabled;
			subscriptionEnabled = gameSettings.subscriptionEnabled;
			leaderboardEnabled = gameSettings.leaderboardEnabled;
		    legacyMarketingEnabled = gameSettings.LegacyMarketingEnabled;

			//Should we save this to prefs?
			//TODO Ahmet
			remotePlayerDataEnabled = gameSettings.RemotePlayerDataEnabled;
		}

        [SerializeField] private string adjustFirstOpenEvent;
		public string AdjustFirstOpenEventToken
        {
            get { return adjustFirstOpenEvent; }
        }

	    [SerializeField] private string adjustFirstLevelCompletedEvent;
	    public string AdjustFirstLevelCompletedEvent
	    {
	        get { return adjustFirstLevelCompletedEvent; }
	    }

	    [SerializeField] private string adjustInterstitialAttemptEvent;
	    public string AdjustInterstitialAttemptEvent
        {
	        get { return adjustInterstitialAttemptEvent; }
	    }

        [SerializeField] private string adjustInterstitialShowEvent;
	    public string AdjustInterstitialShowEvent
        {
	        get { return adjustInterstitialShowEvent; }
	    }

	    [SerializeField] private string adjustRewardedShowEvent;
	    public string AdjustRewardedShowEvent
	    {
	        get { return adjustRewardedShowEvent; }
	    }

        [SerializeField] private string adjustPaymentInitEvent;
		public string AdjustPaymentInitEventToken
        {
            get { return adjustPaymentInitEvent; }
        }

        [SerializeField] private string adjustGiftEvent;
		public string AdjustGiftEventToken
        {
            get { return adjustGiftEvent; }
        }

        [SerializeField] private float minClientVersion;
		public float MinClientVersion
        {
            get { return minClientVersion; }
        }

        [SerializeField] private string appStoreId;
		public string AppStoreId
        {
            get { return appStoreId; }
        }

        [SerializeField] private bool trackProgress;
		public bool TrackProgress
        {
            get { return trackProgress; }
        }

        [SerializeField] private bool pushEnabled;
		public bool PushEnabled
        {
            get { return pushEnabled; }
        }

        [SerializeField] private bool liveTuneEnabled;
		public bool LiveTuneEnabled
        {
            get { return liveTuneEnabled; }
        }

        [SerializeField] private bool subscriptionEnabled;
		public bool SubscriptionEnabled
        {
            get { return subscriptionEnabled; }
        }

        [SerializeField] private bool leaderboardEnabled;
		public bool LeaderboardEnabled
        {
            get { return leaderboardEnabled; }
        }

	    [SerializeField] private bool legacyMarketingEnabled;
	    public bool LegacyMarketingEnabled
	    {
	        get { return legacyMarketingEnabled; }
	    }

		[SerializeField] private bool remotePlayerDataEnabled;
		public bool RemotePlayerDataEnabled
		{
			get { return remotePlayerDataEnabled; }
		}
		
		[SerializeField] private string supportLink;
		public string SupportLink
		{
			get { return supportLink; }
		}
		
		[SerializeField] private string assetBundleBaseUrl;
		public string AssetBundleBaseUrl
		{
			get { return assetBundleBaseUrl; }
		}

        [NonSerialized]
	    public bool PersistentProgressEnabled;

	    public void OverrideRemotePlayerData(bool enabled)
	    {
		    remotePlayerDataEnabled = enabled;
	    }
	}
}
