﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;
namespace RocketGM
{
    public class RocGmSecurity
    {
        public static string Encrypt(string plainText, byte[] key)
        {
            var compressedLength = 0;
            (plainText, compressedLength) = LZ4Utility.CompressString(plainText);

            StringBuilder builder = new StringBuilder();
            builder.Append(compressedLength);
            builder.Append("#");
            builder.Append(plainText);
            plainText = builder.ToString();

            var plainBytes = Encoding.UTF8.GetBytes(plainText);
            string randomBase64String = GenerateRandomBase64String();
            byte[] iv = Encoding.UTF8.GetBytes(randomBase64String);
            byte[] encryptedBytes;
            using (var aes = Aes.Create())
            {
                aes.KeySize = 128;
                aes.Key = key;
                aes.IV = iv;
                aes.Mode = CipherMode.CBC;

                using (var memoryStream = new MemoryStream())
                using (var cryptoStream = new CryptoStream(memoryStream, aes.CreateEncryptor(), CryptoStreamMode.Write))
                {
                    cryptoStream.Write(plainBytes, 0, plainBytes.Length);
                    cryptoStream.FlushFinalBlock();
                    encryptedBytes = memoryStream.ToArray();
                }
            }
            
            builder.Clear();
            builder.Append(randomBase64String);
            builder.Append(Convert.ToBase64String(encryptedBytes));
            return builder.ToString();
        }

        public static string Decrypt(string encryptedBytes, byte[] key)
        {
            SplitIvAndData(encryptedBytes, out var iv, out var data);
            using (var aes = Aes.Create())
            {
                aes.KeySize = 128;
                aes.Key = key;
                aes.IV = iv;
                aes.Mode = CipherMode.CBC;

                using (var memoryStream = new MemoryStream(data))
                using (var cryptoStream = new CryptoStream(memoryStream, aes.CreateDecryptor(), CryptoStreamMode.Read))
                {
                    byte[] decryptedBytes = new byte[data.Length];
                    int decryptedByteCount = cryptoStream.Read(decryptedBytes, 0, decryptedBytes.Length);
                    var result = Encoding.UTF8.GetString(decryptedBytes, 0, decryptedByteCount);
                    SplitDataLengthAndData(result, out var dataString, out var length);
                    return LZ4Utility.DecompressString(dataString, length);
                }
            }
        }

        private static string GenerateRandomBase64String()
        {
            Random random = new Random();
            byte[] bytes = new byte[12];
            random.NextBytes(bytes);
            return Convert.ToBase64String(bytes);
        }

        private static void SplitIvAndData(string encryptedString, out byte[] iv, out byte[] data)
        {
            iv = Encoding.UTF8.GetBytes(encryptedString.AsSpan(0, 16).ToString());
            data = Convert.FromBase64String(encryptedString.AsSpan(16).ToString());
        }

        private static void SplitDataLengthAndData(string encryptedString, out string data, out int length)
        {
            var splitStrings = encryptedString.Split("#");
            length = Convert.ToInt32(splitStrings[0]);
            data = splitStrings[1];
        }
    }
}
